/* This plugin enables gravity votes by clients and auto-responses. 
 * Note: you might want to turn your vote_freq down to 15 or 30 in server.cfg 
 * if your going to have a lot of voting happening (FF, Grav, other ect.)
*/

/* $Id: plugin_gv.sma,v 1.0 2001/04/08 
 * based on yensid's plugin_ff, Cut and pasted :) by BenTheMeek $
 *
 * $Id: plugin_gv.sma,v 1.1 2001/04/18
 * Made script a bit more robust to handle fact admins may manually set the upper gravity to something other than 780.
 * Also changed the text for the "gravity" check to indicate it is just a status.  Would trigger when someone would vote
 * and was confusing.  Also indicates that a vote is not available if one is requested before vote_freq limit was reached.
 * -[MayheM]-
 * $
 * 
 * $Id: plugin_gv.sma,v 1.2 2002/12/15
 * I redid most of the code and make it more readable
 * Added a third option and made the gravity values customizable
 * Fixed lots of typos
 * Made it so the public cannot always access the vote
 * -jtp10181
 * $
*/

#include <core>
#include <console>
#include <string>
#include <admin>
#include <adminlib>

/* Change this to whatever ratio of players need to vote for a setting to change it. */
#define GV_VOTE_RATIO 51

/* Change gravity settings
 * Default Normal is 800
 * Default Low G is 300
 * Default Moon is 130 (800 / 6 = 133.33)*/
 
#define GV_NORM 800
#define GV_LOW 300
#define GV_MOON 130

/* Change to 1 to have a gravity vote start 30 seconds in to a map */
#define AUTO_GV_VOTE 0

/* Access for admin commands and "vote_gravity" say command */
#define ACCESS_ADMIN_VGV ACCESS_KICK
#define ACCESS_VGV 1

new STRING_VERSION[MAX_DATA_LENGTH] = "2.50.50";

public GVVote() {
	new strDummy[10];
	vote("What gravity setting would you like?", "Normal Gravity", "Low Gravity", "Moon Gravity", "HandleGVVote", strDummy);
}

public HandleGVVote(WinningOption,HLData,VoteCount,UserCount) {
	new Text[MAX_TEXT_LENGTH];
	new strNumber[MAX_NUMBER_LENGTH];
	new Ratio = GV_VOTE_RATIO;
	new strData[MAX_DATA_LENGTH];
	convert_string(HLData, strData,MAX_DATA_LENGTH);
	
	if (VoteCount >= Ratio*UserCount/100) {	
		if (WinningOption == 1) {
			if(getvar("sv_gravity") == GV_NORM)
			{
				centersay("Vote Over^nNormal Gravity will stay ON",18,249,244,0);
			}
			else
			{
				centersay("Vote Successful^nChanged to Normal Gravity",18,249,244,0);
				new ExecCommand[MAX_COMMAND_LENGTH];
				snprintf(ExecCommand, MAX_DATA_LENGTH, "sv_gravity %d", GV_NORM );
  				exec(ExecCommand,0);
			}
		}
		else if (WinningOption == 2)
		{
			if(getvar("sv_gravity") == GV_LOW )
			{
				centersay("Vote Over^nLow Gravity will stay ON",18,63,187,239);				
			}
			else
			{
				centersay("Vote Successful^nChanged to Low Gravity",18,63,187,239);
				new ExecCommand[MAX_COMMAND_LENGTH];
				snprintf(ExecCommand, MAX_DATA_LENGTH, "sv_gravity %d", GV_LOW );
  				exec(ExecCommand,0);				
			}
		}
		else if (WinningOption == 3)
		{
			if(getvar("sv_gravity") == GV_MOON )
			{
				centersay("Vote Over^nMoon Gravity will stay ON",18,63,187,239);				
			}
			else
			{
				centersay("Vote Successful^nChanged to Moon Gravity",18,63,187,239);
				new ExecCommand[MAX_COMMAND_LENGTH];
				snprintf(ExecCommand, MAX_DATA_LENGTH, "sv_gravity %d", GV_MOON );
  				exec(ExecCommand,0);				
			}
		}
		else
		{
			centersay("Something weird happened^nGravity will remain the same",18,63,187,239);
		}
	}
	else
	{
		numtostr(Ratio*UserCount/100,strNumber);
		
		if (getvar("sv_gravity") == GV_NORM)
		{
			snprintf(Text, MAX_TEXT_LENGTH, "Gravity vote succeeded, but not enough votes for change (Needed %s)^nNormal Gravity will remain ON", strNumber);
		}
		else if (getvar("sv_gravity") == GV_LOW)
		{
			snprintf(Text, MAX_TEXT_LENGTH, "Gravity vote succeeded, but not enough votes for change (Needed %s)^nLow Gravity will remain ON", strNumber);
		}
		else if (getvar("sv_gravity") == GV_MOON)
		{
			snprintf(Text, MAX_TEXT_LENGTH, "Gravity vote succeeded, but not enough votes for change (Needed %s)^nMoon Gravity will remain ON", strNumber);
		}
		else
		{
			snprintf(Text, MAX_TEXT_LENGTH, "Gravity vote succeeded, but not enough votes for change (Needed %s)^nUnknown Gravity will remain", strNumber);
		}
		centersay(Text,18,63,187,239);
	}
}

public admin_vote_gravity(HLCommand,HLData,HLUserName,UserIndex) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new User[MAX_NAME_LENGTH];
	convert_string(HLCommand,Command,MAX_COMMAND_LENGTH);
	convert_string(HLData,Data,MAX_DATA_LENGTH);
	convert_string(HLUserName,User,MAX_NAME_LENGTH);
	
	if (vote_allowed() != 1)
	{
		messageex(User,"Vote not allowed at this time",print_console);
		return PLUGIN_HANDLED;
	}
	
	say_command(User,Command,Data);		
	GVVote();
	return PLUGIN_HANDLED;
}

public HandleSay(HLCommand,HLData,HLUserName,UserIndex) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new User[MAX_NAME_LENGTH];
	
	convert_string(HLCommand,Command,MAX_COMMAND_LENGTH);
	convert_string(HLData,Data,MAX_DATA_LENGTH);
	convert_string(HLUserName,User,MAX_NAME_LENGTH);

	strstripquotes(Data);

	/* Made pretty cause DOMO said so
	* Added different messages corresponding to the access level of the person
	* Used the new variables and 3 options */
	
	/* If user attempts to start a gravity vote */
	if (strcasecmp(Data, "vote_gravity") == 0)
	{ 
		if (vote_allowed() != 1)
		{
			centersay("^nVote not allowed at this time",7,0,255,0);
		}
		else if (check_auth(ACCESS_ADMIN_VGV) || !adminpresent())
		{
			GVVote();
		}
		else
		{
			messageex(User,"You do not have access to start a vote",print_chat);
		}
	}
	
	/* if user says gravity */
	else if (strcasestr(Data, "gravity")!= -1) {
		if (getvar("sv_gravity") == GV_NORM)
		{
			if (!adminpresent())
			{
				centersay ("Gravity Status:^nNormal Gravity is On!^nTo start a vote say vote_gravity!",7,0,255,0); 
			}
			else
			{
				centersay ("Gravity Status:^nNormal Gravity is On!",7,0,255,0); 
			}
		}
		else if (getvar("sv_gravity") == GV_LOW)
		{
			if (!adminpresent())
			{
				centersay ("Gravity Status:^nLow Gravity is On!^nTo start a vote say vote_gravity!",7,0,255,0); 
			}
			else
			{
				centersay ("Gravity Status:^nLow Gravity is On!",7,0,255,0); 
			}
		}
		else if (getvar("sv_gravity") == GV_MOON)
		{
			if (!adminpresent())
			{
				centersay ("Gravity Status:^nMoon Gravity is On!^nTo start a vote say vote_gravity!",7,0,255,0); 
			}
			else
			{
				centersay ("Gravity Status:^nMoon Gravity is On!",7,0,255,0); 
			}
		}
		else
		{
			if (!adminpresent())
			{
				centersay ("Gravity Status:^nAn Unknown Gravity is Set^nTo start a vote say vote_gravity!",7,0,255,0); 
			}
			else
			{
				centersay ("Gravity Status:^nAn Unknown Gravity is Set",7,0,255,0); 
			}
		}
	}

	return PLUGIN_CONTINUE;
}

public adminpresent() {

	new i;
	new maxplayers = maxplayercount();
	new Name[MAX_NAME_LENGTH];
	new SessionID;
	new Team;
	new WONID;

	for(i=1; i<=maxplayers; i++) {
		strinit(Name);
		if(playerinfo(i,Name,MAX_NAME_LENGTH,SessionID,WONID,Team)==1) {
			if(access(ACCESS_ADMIN_VGV,Name)!=0) {
				return TRUE;
			}
		}
	}
	return FALSE;
}

public plugin_init() {
	plugin_registerinfo("Gravity Plugin","Enables Gravity Vote Normal/Low/Moon and Auto-Responses.",STRING_VERSION);
	plugin_registercmd("admin_vote_gravity","admin_vote_gravity",ACCESS_ADMIN_VGV,"admin_vote_gravity: Starts a gravity vote.");
	plugin_registercmd("say","HandleSay",ACCESS_ALL);
	plugin_registerhelp("say",ACCESS_ALL,"say gravity: Will give gravity status.");
	plugin_registerhelp("say",ACCESS_VGV,"say vote_gravity: Will start vote for gravity.");

#if AUTO_GV_VOTE == 1	
	set_timer("GVVote",30,1);
#endif

	return PLUGIN_CONTINUE;
}
